@@ -22,6 +22,7 @@
 #include <string.h>
 #include <assert.h>
 #include <errno.h>
+#include <locale.h>
 #include "cmd_parser.h"
 #include "deltarpms.h"
 #include "error.h"
@@ -65,9 +66,30 @@ struct CmdOptions _cmd_options = {
         .zck_compression            = FALSE,
         .zck_dict_dir               = NULL,
         .recycle_pkglist            = FALSE,
+
+        .keep_all_metadata          = TRUE,
+        .nevra_duplicates           = CR_ARG_DUP_NEVRA_KEEP_ALL,
     };
 
 
+gboolean
+duplicated_nevra_option_parser(const gchar *,
+                               const gchar *value,
+                               gpointer,
+                               GError **error)
+{
+    if (!g_strcmp0(value, "keep"))
+        _cmd_options.nevra_duplicates = CR_ARG_DUP_NEVRA_KEEP_ALL;
+    else if (!g_strcmp0(value, "keep-last"))
+        _cmd_options.nevra_duplicates = CR_ARG_DUP_NEVRA_KEEP_LAST;
+    else {
+        g_set_error(error, ERR_DOMAIN, CRE_BADARG,
+                    "Bad --duplicated-nevra argument, use 'keep' or 'keep-last'.");
+        return FALSE;
+    }
+    return TRUE;
+}
+
 
 // Command line params
 
@@ -97,6 +119,8 @@ static GOptionEntry cmd_entries[] =
       "Generate sqlite databases for use with yum.", NULL },
     { "no-database", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.no_database),
       "Do not generate sqlite databases in the repository.", NULL },
+    { "filelists-ext", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.filelists_ext),
+      "Create filelists-ext metadata with file hashes.", NULL },
     { "update", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.update),
       "If metadata already exists in the outputdir and an rpm is unchanged "
       "(based on file size and mtime) since the metadata was generated, reuse "
@@ -104,7 +128,8 @@ static GOptionEntry cmd_entries[] =
       "large repository with only a few new or modified rpms "
       "this can significantly reduce I/O and processing time.", NULL },
     { "update-md-path", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &(_cmd_options.update_md_paths),
-      "Use the existing repodata for --update from this path.", NULL },
+      "Existing metadata from this path are loaded and reused in addition to those "
+      "present in the outputdir (works only with --update). Can be specified multiple times.", NULL },
     { "skip-stat", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.skip_stat),
       "Skip the stat() call on a --update, assumes if the filename is the same "
       "then the file is still the same (only use this if you're fairly "
@@ -146,7 +171,7 @@ static GOptionEntry cmd_entries[] =
       "User-specified revision for this repository.", "REVISION" },
     { "set-timestamp-to-revision", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.set_timestamp_to_revision),
       "Set timestamp fields in repomd.xml and last modification times of created repodata to a value given with --revision. "
-      "This requires --revision to be a timestamp formated in 'date +%s' format.", NULL },
+      "This requires --revision to be a timestamp formatted in 'date +%s' format.", NULL },
     { "read-pkgs-list", 0, 0, G_OPTION_ARG_FILENAME, &(_cmd_options.read_pkgs_list),
       "Output the paths to the pkgs actually read useful with --update.",
       "READ_PKGS_LIST" },
@@ -167,6 +192,9 @@ static GOptionEntry cmd_entries[] =
 #endif
     { "keep-all-metadata", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.keep_all_metadata),
       "Keep all additional metadata (not primary, filelists and other xml or sqlite files, "
+      "nor their compressed variants) from source repository during update (default).", NULL },
+    { "discard-additional-metadata", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.discard_additional_metadata),
+      "Discard all additional metadata (not primary, filelists and other xml or sqlite files, "
       "nor their compressed variants) from source repository during update.", NULL },
     { "compatibility", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.compatibility),
       "Enforce maximal compatibility with classical createrepo (Affects only: --retain-old-md).", NULL },
@@ -202,11 +230,13 @@ static GOptionEntry cmd_entries[] =
     { "repomd-checksum", 0, 0, G_OPTION_ARG_STRING, &(_cmd_options.repomd_checksum),
       "Checksum type to be used in repomd.xml", "CHECKSUM_TYPE"},
     { "error-exit-val", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.error_exit_val),
-      "Exit with retval 2 if there were any errors during processing", NULL },
+      "Exit with retval 2 if there were any errors during processing (option deprecated, on by default)", NULL },
     { "recycle-pkglist", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.recycle_pkglist),
       "Read the list of packages from old metadata directory and re-use it.  This "
       "option is only useful with --update (complements --pkglist and friends).",
       NULL },
+    { "duplicated-nevra", 0, 0, G_OPTION_ARG_CALLBACK, duplicated_nevra_option_parser,
+      "What to do about duplicates.", NULL, },
     { NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL },
 };
 
@@ -216,12 +246,12 @@ static GOptionEntry expert_entries[] =
     { "ignore-lock", 0, 0, G_OPTION_ARG_NONE, &(_cmd_options.ignore_lock),
       "Expert (risky) option: Ignore an existing .repodata/. "
       "(Remove the existing .repodata/ and create an empty new one "
-      "to serve as a lock for other createrepo intances. For the repodata "
+      "to serve as a lock for other createrepo instances. For the repodata "
       "generation, a different temporary dir with the name in format "
       "\".repodata.time.microseconds.pid/\" will be used). "
       "NOTE: Use this option on your "
       "own risk! If two createrepos run simultaneously, then the state of the "
-      "generated metadata is not guaranted - it can be inconsistent and wrong.",
+      "generated metadata is not guaranteed - it can be inconsistent and wrong.",
       NULL },
     { NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL },
 };
@@ -248,6 +278,13 @@ struct CmdOptions *parse_arguments(int *argc, char ***argv, GError **err)
     g_option_group_add_entries(group_expert, expert_entries);
     g_option_context_add_group(context, group_expert);
 
+    /*
+      setlocale is required to ensure that NON-ASCII chars can be parsed
+      SEE https://developer-old.gnome.org/glib/unstable/glib-running.html#setlocale
+      for further information
+    */
+    setlocale(LC_ALL, "");
+
     ret = g_option_context_parse(context, argc, argv, err);
     g_option_context_free(context);
 
@@ -509,9 +546,13 @@ check_arguments(struct CmdOptions *options,
         x++;
     }
 
-    // Check keep-all-metadata
-    if (options->keep_all_metadata && !options->update) {
-        g_warning("--keep-all-metadata has no effect (--update is not used)");
+    if (options->discard_additional_metadata) {
+        options->keep_all_metadata = FALSE;
+    }
+
+    // Check discard-additional-metadata
+    if (options->discard_additional_metadata && !options->update) {
+        g_warning("--discard-additional-metadata has no effect (--update is not used)");
     }
 
     // Process --distro tags
@@ -589,7 +630,7 @@ check_arguments(struct CmdOptions *options,
     // Check cut_dirs
     if (options->cut_dirs < 0) {
         g_set_error(err, ERR_DOMAIN, CRE_BADARG,
-                    "--cur-dirs value must be possitive integer");
+                    "--cur-dirs value must be positive integer");
         return FALSE;
     }
 
@@ -636,5 +677,6 @@ free_options(struct CmdOptions *options)
     cr_slist_free_full(options->l_update_md_paths, g_free);
     cr_slist_free_full(options->distro_cpeids, g_free);
     cr_slist_free_full(options->distro_values, g_free);
+    cr_slist_free_full(options->modulemd_metadata, g_free);
     g_slist_free(options->oldpackagedirs_paths);
 }
