@@ -86,11 +86,6 @@ LZMA_CHECK_SHA256
 #define XZ_DECODER_FLAGS        0
 #define XZ_BUFFER_SIZE          (1024*32)
 
-#if ZLIB_VERNUM < 0x1240
-// XXX: Zlib has gzbuffer since 1.2.4
-#define gzbuffer(a,b) 0
-#endif
-
 cr_ContentStat *
 cr_contentstat_new(cr_ChecksumType type, GError **err)
 {
@@ -191,7 +186,6 @@ cr_detect_compression(const char *filename, GError **err)
             g_str_has_prefix(mime_type, "application/gzipped") ||
             g_str_has_prefix(mime_type, "application/x-gzip-compressed") ||
             g_str_has_prefix(mime_type, "application/x-compress") ||
-            g_str_has_prefix(mime_type, "application/x-gzip") ||
             g_str_has_prefix(mime_type, "application/x-gunzip") ||
             g_str_has_prefix(mime_type, "multipart/x-gzip"))
         {
@@ -304,10 +298,12 @@ cr_cktype_from_zck(zckCtx *zck, GError **err)
                     "Unable to read hash from zchunk file");
         return CR_CHECKSUM_UNKNOWN;
     }
-    if (cktype == ZCK_HASH_SHA1)
-        return CR_CHECKSUM_SHA1;
-    else if (cktype == ZCK_HASH_SHA256)
+    if (cktype == ZCK_HASH_SHA256)
         return CR_CHECKSUM_SHA256;
+#ifdef WITH_LEGACY_HASHES
+    else if (cktype == ZCK_HASH_SHA1)
+        return CR_CHECKSUM_SHA1;
+#endif
     else {
         const char *ckname = zck_hash_name_from_type(cktype);
         if (ckname == NULL)
@@ -591,8 +587,6 @@ cr_sopen(const char *filename,
         case (CR_CW_ZCK_COMPRESSION): { // -------------------------------------
 #ifdef WITH_ZCHUNK
             FILE *f = fopen(filename, mode_str);
-            file->INNERFILE = f;
-            int fd = fileno(f);
 
             if (!f) {
                 g_set_error(err, ERR_DOMAIN, CRE_IO,
@@ -600,6 +594,9 @@ cr_sopen(const char *filename,
                 break;
             }
 
+            file->INNERFILE = f;
+            int fd = fileno(f);
+
             file->FILE = (void *) zck_create();
             zckCtx *zck = file->FILE;
             if (mode == CR_CW_MODE_WRITE) {
@@ -608,14 +605,12 @@ cr_sopen(const char *filename,
                     zck_set_log_fd(STDOUT_FILENO);
                     g_set_error(err, ERR_DOMAIN, CRE_IO, "%s",
                                 zck_get_error(zck));
-                    g_free(file);
                     break;
                 }
             } else {
                 if (!file->FILE || !zck_init_read(zck, fd)) {
                     g_set_error(err, ERR_DOMAIN, CRE_IO,
                                 "%s", zck_get_error(zck));
-                    g_free(file);
                     break;
                 }
             }
@@ -790,7 +785,7 @@ cr_close(CR_FILE *cr_file, GError **err)
 
                 switch (rc) {
                     case BZ_SEQUENCE_ERROR:
-                        // This really shoud not happen
+                        // This really should not happen
                         err_msg = "file was opened with BZ2_bzReadOpen";
                         break;
                     case BZ_IO_ERROR:
@@ -887,12 +882,14 @@ cr_close(CR_FILE *cr_file, GError **err)
                     ret = CRE_ZCK;
                     g_set_error(err, ERR_DOMAIN, CRE_ZCK,
                         "Unable to end final chunk: %s", zck_get_error(zck));
+                    break;
                 }
             }
             if (!zck_close(zck)) {
                 ret = CRE_ZCK;
                 g_set_error(err, ERR_DOMAIN, CRE_ZCK,
                         "Unable to close zchunk file: %s", zck_get_error(zck));
+                break;
             }
             cr_ChecksumType cktype = cr_cktype_from_zck(zck, err);
             if (cktype == CR_CHECKSUM_UNKNOWN) {
@@ -951,7 +948,7 @@ int
 cr_read(CR_FILE *cr_file, void *buffer, unsigned int len, GError **err)
 {
     int bzerror;
-    int ret;
+    int ret = CR_CW_ERR;
 
     assert(cr_file);
     assert(buffer);
@@ -995,11 +992,11 @@ cr_read(CR_FILE *cr_file, void *buffer, unsigned int len, GError **err)
 
                 switch (bzerror) {
                     case BZ_PARAM_ERROR:
-                        // This shoud not happend
+                        // This should not happend
                         err_msg = "bad function params!";
                         break;
                     case BZ_SEQUENCE_ERROR:
-                        // This shoud not happend
+                        // This should not happend
                         err_msg = "file was opened with BZ2_bzWriteOpen";
                         break;
                     case BZ_IO_ERROR:
@@ -1227,11 +1224,11 @@ cr_write(CR_FILE *cr_file, const void *buffer, unsigned int len, GError **err)
 
                 switch (bzerror) {
                     case BZ_PARAM_ERROR:
-                        // This shoud not happend
+                        // This should not happend
                         err_msg = "bad function params!";
                         break;
                     case BZ_SEQUENCE_ERROR:
-                        // This shoud not happend
+                        // This should not happend
                         err_msg = "file was opened with BZ2_bzReadOpen";
                         break;
                     case BZ_IO_ERROR:
@@ -1512,6 +1509,7 @@ cr_printf(GError **err, CR_FILE *cr_file, const char *format, ...)
         g_debug("%s: vasprintf() call failed", __func__);
         g_set_error(err, ERR_DOMAIN, CRE_MEMORY,
                     "vasprintf() call failed");
+        g_free(buf);
         return CR_CW_ERR;
     }
 
@@ -1546,7 +1544,7 @@ cr_printf(GError **err, CR_FILE *cr_file, const char *format, ...)
     return ret;
 }
 
-ssize_t 
+ssize_t
 cr_get_zchunk_with_index(CR_FILE *cr_file, ssize_t zchunk_index, char **copy_buf, GError **err)
 {
     assert(cr_file);
