@@ -21,6 +21,7 @@
 
 #include <glib/gstdio.h>
 #include <glib.h>
+#include <gio/gio.h>
 #include <arpa/inet.h>
 #include <assert.h>
 #include <curl/curl.h>
@@ -31,6 +32,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/time.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <time.h>
 #include <unistd.h>
 #include "cleanup.h"
@@ -259,10 +262,11 @@ cr_get_header_byte_range(const char *filename, GError **err)
 
     FILE *fp = fopen(filename, "rb");
     if (!fp) {
+        const gchar * fopen_error = g_strerror(errno);
         g_debug("%s: Cannot open file %s (%s)", __func__, filename,
-                g_strerror(errno));
+                fopen_error);
         g_set_error(err, ERR_DOMAIN, CRE_IO,
-                    "Cannot open %s: %s", filename, g_strerror(errno));
+                    "Cannot open %s: %s", filename, fopen_error);
         return results;
     }
 
@@ -270,10 +274,10 @@ cr_get_header_byte_range(const char *filename, GError **err)
     // Get header range
 
     if (fseek(fp, 104, SEEK_SET) != 0) {
-        g_debug("%s: fseek fail on %s (%s)", __func__, filename,
-                g_strerror(errno));
+        const gchar * fseek_error = g_strerror(errno);
+        g_debug("%s: fseek fail on %s (%s)", __func__, filename, fseek_error);
         g_set_error(err, ERR_DOMAIN, CRE_IO,
-                    "Cannot seek over %s: %s", filename, g_strerror(errno));
+                    "Cannot seek over %s: %s", filename, fseek_error);
         fclose(fp);
         return results;
     }
@@ -404,19 +408,21 @@ cr_copy_file(const char *src, const char *in_dst, GError **err)
 
     // Open src file
     if ((orig = fopen(src, "rb")) == NULL) {
+        const gchar * fopen_error = g_strerror(errno);
         g_debug("%s: Cannot open source file %s (%s)", __func__, src,
-                g_strerror(errno));
+                fopen_error);
         g_set_error(err, ERR_DOMAIN, CRE_IO,
-                    "Cannot open file %s: %s", src, g_strerror(errno));
+                    "Cannot open file %s: %s", src, fopen_error);
         return FALSE;
     }
 
     // Open dst file
     if ((new = fopen(dst, "wb")) == NULL) {
+        const gchar * fopen_error = g_strerror(errno);
         g_debug("%s: Cannot open destination file %s (%s)", __func__, dst,
-                g_strerror(errno));
+                fopen_error);
         g_set_error(err, ERR_DOMAIN, CRE_IO,
-                    "Cannot open file %s: %s", dst, g_strerror(errno));
+                    "Cannot open file %s: %s", dst, fopen_error);
         return FALSE;
     }
 
@@ -429,10 +435,11 @@ cr_copy_file(const char *src, const char *in_dst, GError **err)
         }
 
         if (fwrite(buf, 1, readed, new) != readed) {
+            const gchar * fwrite_error = g_strerror(errno);
             g_debug("%s: Error while copy %s -> %s (%s)", __func__, src,
-                    dst, g_strerror(errno));
+                    dst, fwrite_error);
             g_set_error(err, ERR_DOMAIN, CRE_IO,
-                    "Error while write %s: %s", dst, g_strerror(errno));
+                    "Error while write %s: %s", dst, fwrite_error);
             return FALSE;
         }
     }
@@ -444,7 +451,7 @@ cr_copy_file(const char *src, const char *in_dst, GError **err)
 
 int
 cr_compress_file_with_stat(const char *src,
-                           char **in_dst,
+                           const char *in_dst,
                            cr_CompressionType compression,
                            cr_ContentStat *stat,
                            const char *zck_dict_dir,
@@ -456,7 +463,7 @@ cr_compress_file_with_stat(const char *src,
     char buf[BUFFER_SIZE];
     CR_FILE *orig = NULL;
     CR_FILE *new = NULL;
-    gchar *dst = (gchar *) *in_dst;
+    gchar *dst = (gchar *) in_dst;
     GError *tmp_err = NULL;
 
     assert(src);
@@ -474,30 +481,23 @@ cr_compress_file_with_stat(const char *src,
 
     if (!dst) {
         // If destination is NULL, use src + compression suffix
-        *in_dst = g_strconcat(src,
-                              c_suffix,
-                              NULL);
+        dst = g_strconcat(src, c_suffix, NULL);
     } else if (g_str_has_suffix(dst, "/")) {
         // If destination is dir use filename from src + compression suffix
-        *in_dst = g_strconcat(dst,
-                              cr_get_filename(src),
-                              c_suffix,
-                              NULL);
+        dst = g_strconcat(dst, cr_get_filename(src), c_suffix, NULL);
     } else if (c_suffix && !g_str_has_suffix(dst, c_suffix)) {
-        cr_CompressionType old_type = cr_detect_compression(src, &tmp_err);
+        // If destination is missing compression suffix or has a different one, use specified compression suffix
+        cr_CompressionType old_type = cr_detect_compression(dst, &tmp_err);
         if (tmp_err) {
-            g_debug("%s: Unable to detect compression type of %s", __func__, src);
+            g_debug("%s: Unable to detect compression type of %s, using the filename as is.", __func__, dst);
             g_clear_error(&tmp_err);
-        } else if (old_type != CR_CW_NO_COMPRESSION) {
+        } else if (old_type == CR_CW_NO_COMPRESSION) {
+            dst = g_strconcat(dst, c_suffix, NULL);
+        } else {
             _cleanup_free_ gchar *tmp_file = g_strndup(dst, strlen(dst) - strlen(cr_compression_suffix(old_type)));
-            *in_dst = g_strconcat(tmp_file,
-                                  c_suffix,
-                                  NULL);
+            dst = g_strconcat(tmp_file, c_suffix, NULL);
         }
     }
-    if (dst != *in_dst && dst)
-        g_free(dst);
-    dst = (gchar *) *in_dst;
 
     int mode = CR_CW_AUTO_DETECT_COMPRESSION;
 
@@ -508,6 +508,8 @@ cr_compress_file_with_stat(const char *src,
     if (!orig) {
         ret = tmp_err->code;
         g_propagate_prefixed_error(err, tmp_err, "Cannot open %s: ", src);
+        if (dst != in_dst)
+            g_free(dst);
         return ret;
     }
 
@@ -534,6 +536,7 @@ cr_compress_file_with_stat(const char *src,
             g_set_error(err, ERR_DOMAIN, CRE_IO,
                         "Error reading zchunk dict %s: %s",
                         dict_file, tmp_err->message);
+            g_clear_error(&tmp_err);
             ret = CRE_IO;
             goto compress_file_cleanup;
         }
@@ -569,6 +572,9 @@ cr_compress_file_with_stat(const char *src,
 
 compress_file_cleanup:
 
+    if (dst != in_dst)
+        g_free(dst);
+
     if (orig)
         cr_close(orig, NULL);
 
@@ -621,6 +627,12 @@ cr_decompress_file_with_stat(const char *src,
 
     if (!in_dst || g_str_has_suffix(in_dst, "/")) {
         char *filename = cr_get_filename(src);
+        if (!filename) {
+            g_debug("%s: Cannot get filename from: %s", __func__, src);
+            g_set_error(err, ERR_DOMAIN, CRE_NOFILE,
+                        "Cannot get filename from: %s", src);
+            return CRE_NOFILE;
+        }
         if (g_str_has_suffix(filename, c_suffix)) {
             filename = g_strndup(filename, strlen(filename) - strlen(c_suffix));
         } else {
@@ -651,10 +663,11 @@ cr_decompress_file_with_stat(const char *src,
 
     new = fopen(dst, "wb");
     if (!new) {
+        const gchar * fopen_error = g_strerror(errno);
         g_debug("%s: Cannot open destination file %s (%s)",
-                __func__, dst, g_strerror(errno));
+                __func__, dst, fopen_error);
         g_set_error(err, ERR_DOMAIN, CRE_IO,
-                    "Cannot open %s: %s", src, g_strerror(errno));
+                    "Cannot open %s: %s", src, fopen_error);
         ret = CRE_IO;
         goto compress_file_cleanup;
     }
@@ -670,10 +683,11 @@ cr_decompress_file_with_stat(const char *src,
         }
 
         if (fwrite(buf, 1, readed, new) != (size_t) readed) {
+            const gchar * fwrite_error = g_strerror(errno);
             g_debug("%s: Error while copy %s -> %s (%s)",
-                    __func__, src, dst, g_strerror(errno));
+                    __func__, src, dst, fwrite_error);
             g_set_error(err, ERR_DOMAIN, CRE_IO,
-                        "Error while write %s: %s", dst, g_strerror(errno));
+                        "Error while write %s: %s", dst, fwrite_error);
             ret = CRE_IO;
             goto compress_file_cleanup;
         }
@@ -734,6 +748,7 @@ cr_download(CURL *in_handle,
     errorbuf[0] = '\0';
     rcode = curl_easy_setopt(handle, CURLOPT_ERRORBUFFER, errorbuf);
     if (rcode != CURLE_OK) {
+        curl_easy_cleanup(handle);
         g_set_error(err, ERR_DOMAIN, CRE_CURL,
                     "curl_easy_setopt failed(CURLOPT_ERRORBUFFER): %s",
                     curl_easy_strerror(rcode));
@@ -743,6 +758,7 @@ cr_download(CURL *in_handle,
     // Set URL
     rcode = curl_easy_setopt(handle, CURLOPT_URL, url);
     if (rcode != CURLE_OK) {
+        curl_easy_cleanup(handle);
         g_set_error(err, ERR_DOMAIN, CRE_CURL,
                     "curl_easy_setopt failed(CURLOPT_URL): %s",
                     curl_easy_strerror(rcode));
@@ -753,6 +769,7 @@ cr_download(CURL *in_handle,
     // Set output file descriptor
     rcode = curl_easy_setopt(handle, CURLOPT_WRITEDATA, file);
     if (rcode != CURLE_OK) {
+        curl_easy_cleanup(handle);
         g_set_error(err, ERR_DOMAIN, CRE_CURL,
                     "curl_easy_setopt(CURLOPT_WRITEDATA) failed: %s",
                     curl_easy_strerror(rcode));
@@ -763,6 +780,7 @@ cr_download(CURL *in_handle,
     // Download the file
     rcode = curl_easy_perform(handle);
     if (rcode != CURLE_OK) {
+        curl_easy_cleanup(handle);
         g_set_error(err, ERR_DOMAIN, CRE_CURL,
                     "curl_easy_perform failed: %s: %s",
                     curl_easy_strerror(rcode), errorbuf);
@@ -770,13 +788,13 @@ cr_download(CURL *in_handle,
         return CRE_CURL;
     }
 
+    curl_easy_cleanup(handle);
+
     g_debug("%s: Successfully downloaded: %s", __func__, dst);
 
     return CRE_OK;
 }
 
-
-
 gboolean
 cr_better_copy_file(const char *src, const char *in_dst, GError **err)
 {
@@ -801,7 +819,6 @@ cr_better_copy_file(const char *src, const char *in_dst, GError **err)
     return TRUE;
 }
 
-
 int
 cr_remove_dir_cb(const char *fpath,
                  G_GNUC_UNUSED const struct stat *sb,
@@ -833,6 +850,16 @@ cr_remove_dir(const char *path, GError **err)
     return CRE_OK;
 }
 
+gboolean
+cr_move_recursive(const char *srcDir, const char *dstDir, GError **err)
+{
+    if (rename(srcDir, dstDir) == -1) {
+        if (!cr_gio_cp(g_file_new_for_path(srcDir), g_file_new_for_path(dstDir), G_FILE_COPY_ALL_METADATA, NULL, err))
+            return FALSE;
+        return (cr_remove_dir(srcDir, err) == CRE_OK);
+    }
+    return TRUE;
+}
 
 // Return path with exactly one trailing '/'
 char *
@@ -918,10 +945,7 @@ cr_str_to_version(const char *str)
         // Whole string has been converted successfully
         return ver;
     } else {
-        if (endptr[0] == '.') {
-            // '.' is supposed to be delimiter -> skip it and go to next chunk
-            ptr = endptr+1;
-        } else {
+        if (endptr[0] != '.') { // '.' is supposed to be delimiter
             ver.suffix = g_strdup(endptr);
             return ver;
         }
@@ -1060,6 +1084,10 @@ cr_split_rpm_filename(const char *filename)
 
     nevra = cr_str_to_nevra(str);
     g_free(str);
+    if (!nevra) {
+        g_free(epoch);
+        return NULL;
+    }
 
     if (epoch) {
         g_free(nevra->epoch);
@@ -1136,7 +1164,6 @@ cr_str_to_nevr(const char *instr)
         if (nvr[i] == '-') {
             nevr->version = g_strdup(nvr+i+1);
             nvr[i] = '\0';
-            len = i;
             break;
         }
 
@@ -1180,7 +1207,6 @@ cr_str_to_nevra(const char *instr)
     cr_NEVR *nevr;
     cr_NEVRA *nevra = NULL;
     gchar *str, *epoch = NULL;
-    size_t len;
     int i;
 
     if (!instr)
@@ -1199,20 +1225,18 @@ cr_str_to_nevra(const char *instr)
         {
             // Strip epoch from the very end
             epoch = epoch_candidate;
+            g_free(str);
             str = nvra_epoch[0];
         } else {
             g_strfreev(nvra_epoch);
         }
     }
 
-    len = strlen(str);
-
     // Get arch
-    for (i = len-1; i >= 0; i--)
+    for (i = strlen(str)-1; i >= 0; i--)
         if (str[i] == '.') {
             nevra->arch = g_strdup(str+i+1);
             str[i] = '\0';
-            len = i;
             break;
         }
 
@@ -1220,10 +1244,19 @@ cr_str_to_nevra(const char *instr)
         g_warning("Invalid arch %s", nevra->arch);
         cr_nevra_free(nevra);
         g_free(str);
+        g_free(epoch);
         return NULL;
     }
 
     nevr = cr_str_to_nevr(str);
+    if (!nevr) {
+        g_warning("Invalid nevr %s", str);
+        cr_nevra_free(nevra);
+        g_free(str);
+        g_free(epoch);
+        return NULL;
+    }
+
     nevra->name     = nevr->name;
     nevra->epoch    = nevr->epoch;
     nevra->version  = nevr->version;
@@ -1390,6 +1423,44 @@ cr_cp(const char *src,
     return ret;
 }
 
+gboolean
+cr_gio_cp(GFile *src,
+      GFile *dst,
+      GFileCopyFlags flags,
+      GCancellable *cancellable,
+      GError **err)
+{
+    assert(src);
+    assert(dst);
+    assert(!err || *err == NULL);
+
+    GFileType type = g_file_query_file_type(src, G_FILE_QUERY_INFO_NONE, NULL);
+
+    if (type == G_FILE_TYPE_DIRECTORY) {
+        g_file_make_directory(dst, cancellable, err);
+        g_file_copy_attributes(src, dst, flags, cancellable, err);
+
+        GFileEnumerator *enumerator = g_file_enumerate_children(src, G_FILE_ATTRIBUTE_STANDARD_NAME, G_FILE_QUERY_INFO_NONE, cancellable, err);
+        for (GFileInfo *info = g_file_enumerator_next_file(enumerator, cancellable, err); info != NULL; info = g_file_enumerator_next_file(enumerator, cancellable, err)) {
+            const char *relative_path = g_file_info_get_name(info);
+            cr_gio_cp(
+                g_file_resolve_relative_path(src, relative_path),
+                g_file_resolve_relative_path(dst, relative_path),
+                flags, cancellable, err);
+        }
+    } else if (type == G_FILE_TYPE_REGULAR) {
+        g_file_copy(src, dst, flags, cancellable, NULL, NULL, err);
+    }
+
+    if (err != NULL) {
+        return TRUE;
+    }
+    else {
+        return FALSE;
+    }
+
+}
+
 gboolean
 cr_rm(const char *path,
       cr_RmFlags flags,
@@ -1583,7 +1654,7 @@ cr_get_dict_file(const gchar *dir, const gchar *file)
     free(dict_file);
     if (!g_file_test(full_path, G_FILE_TEST_EXISTS)) {
         g_warning("%s: Zchunk dict %s doesn't exist", __func__, full_path);
-        free(full_path);
+        g_free(full_path);
         return NULL;
     }
     return full_path;
